@@ -1,26 +1,31 @@
 from . import distribution
 from . import geometry
 from .distribution import Distribution
-from .expr import Expr, Node
+from .expr import Expr, Node, T
 from .geometry import Point, Segment
-from typing import *
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    FrozenSet,
+    Generic,
+    Iterator,
+    List,
+    Optional,
+    Set,
+    Tuple,
+)
 import collections
 import datetime
 import itertools
 import math
-import numpy as np
 import pulp
-
-
-T = TypeVar('T')
+import random
 
 LOAD = 'load'
 NETWORK = 'network'
 LATENCY = 'latency'
 
-# TODO(mwhittaker): Add some other non-optimal strategies.
-# TODO(mwhittaker): Make it easy to make arbitrary strategies.
-
 
 class NoStrategyFoundError(ValueError):
     pass
@@ -146,7 +151,7 @@ class QuorumSystem(Generic[T]):
                 latency_limit: Optional[datetime.timedelta] = None,
                 read_fraction: Optional[Distribution] = None,
                 write_fraction: Optional[Distribution] = None,
-                f: int = 0) -> float:
+                f: int = 0) -> datetime.timedelta:
         return self.strategy(
             optimize=optimize,
             load_limit=load_limit,
@@ -171,7 +176,7 @@ class QuorumSystem(Generic[T]):
         else:
             xs = list(self.elements())
             read_quorums = list(self._f_resilient_quorums(f, xs, self.reads))
-            write_quorums = list(self._f_resilient_quorums(f, xs, self.reads))
+            write_quorums = list(self._f_resilient_quorums(f, xs, self.writes))
             if len(read_quorums) == 0:
                 raise NoStrategyFoundError(
                     f'There are no {f}-resilient read quorums')
@@ -369,10 +374,10 @@ class QuorumSystem(Generic[T]):
         node to be less than l. Using the example above, we have
 
                min l subject to
-               fr * r0 * rcap(a) + fw * (w0 + w1) * wcap(a) <= l
-               fr * r0 * rcap(b) + fw * (w2 + w3) * wcap(b) <= l
-               fr * r1 * rcap(c) + fw * (w0 + w2) * wcap(c) <= l
-               fr * r1 * rcap(d) + fw * (w1 + w3) * wcap(d) <= l
+               fr * r0 / rcap(a) + fw * (w0 + w1) / wcap(a) <= l
+               fr * r0 / rcap(b) + fw * (w2 + w3) / wcap(b) <= l
+               fr * r1 / rcap(c) + fw * (w0 + w2) / wcap(c) <= l
+               fr * r1 / rcap(d) + fw * (w1 + w3) / wcap(d) <= l
 
         To compute the load of a strategy with respect to a distribution of
         read_fractions, we compute the load for every value of fr and weight
@@ -380,14 +385,14 @@ class QuorumSystem(Generic[T]):
         the time and 0.5 20% of the time. We have:
 
                min 0.8 * l0.9 + 0.2 * l0.5
-               0.9 * r0 * rcap(a) + 0.1 * (w0 + w1) * wcap(a) <= l0.9
-               0.9 * r0 * rcap(b) + 0.1 * (w2 + w3) * wcap(b) <= l0.9
-               0.9 * r1 * rcap(c) + 0.1 * (w0 + w2) * wcap(c) <= l0.9
-               0.9 * r1 * rcap(d) + 0.1 * (w1 + w3) * wcap(d) <= l0.9
-               0.5 * r0 * rcap(a) + 0.5 * (w0 + w1) * wcap(a) <= l0.5
-               0.5 * r0 * rcap(b) + 0.5 * (w2 + w3) * wcap(b) <= l0.5
-               0.5 * r1 * rcap(c) + 0.5 * (w0 + w2) * wcap(c) <= l0.5
-               0.5 * r1 * rcap(d) + 0.5 * (w1 + w3) * wcap(d) <= l0.5
+               0.9 * r0 / rcap(a) + 0.1 * (w0 + w1) / wcap(a) <= l0.9
+               0.9 * r0 / rcap(b) + 0.1 * (w2 + w3) / wcap(b) <= l0.9
+               0.9 * r1 / rcap(c) + 0.1 * (w0 + w2) / wcap(c) <= l0.9
+               0.9 * r1 / rcap(d) + 0.1 * (w1 + w3) / wcap(d) <= l0.9
+               0.5 * r0 / rcap(a) + 0.5 * (w0 + w1) / wcap(a) <= l0.5
+               0.5 * r0 / rcap(b) + 0.5 * (w2 + w3) / wcap(b) <= l0.5
+               0.5 * r1 / rcap(c) + 0.5 * (w0 + w2) / wcap(c) <= l0.5
+               0.5 * r1 / rcap(d) + 0.5 * (w1 + w3) / wcap(d) <= l0.5
 
         Let the expression for load be LOAD.
 
@@ -438,10 +443,10 @@ class QuorumSystem(Generic[T]):
         Using the example above assuming a fixed fr, we have:
 
                min l subject to
-               fr * r0 * rcap(a) + fw * (w0 + w1) * wcap(a) <= l
-               fr * r0 * rcap(b) + fw * (w2 + w3) * wcap(b) <= l
-               fr * r1 * rcap(c) + fw * (w0 + w2) * wcap(c) <= l
-               fr * r1 * rcap(d) + fw * (w1 + w3) * wcap(d) <= l
+               fr * r0 / rcap(a) + fw * (w0 + w1) / wcap(a) <= l
+               fr * r0 / rcap(b) + fw * (w2 + w3) / wcap(b) <= l
+               fr * r1 / rcap(c) + fw * (w0 + w2) / wcap(c) <= l
+               fr * r1 / rcap(d) + fw * (w1 + w3) / wcap(d) <= l
                fr * (2*r0 + 2*r1) + fw * (2*w0 + 2*w1 + 2*w2 + 2*w3) <= 2
                fr * (1*r0 + 1*r1) + fw * (1*w0 + 1*w1 + 1*w2 + 1*w3) <= 3
 
@@ -450,10 +455,10 @@ class QuorumSystem(Generic[T]):
 
                min fr * (2*r0 + 2*r1) +
                    fw * (2*w0 + 2*w1 + 2*w2 + 2*w3) subject to
-               fr * r0 * rcap(a) + fw * (w0 + w1) * wcap(a) <= 4
-               fr * r0 * rcap(b) + fw * (w2 + w3) * wcap(b) <= 4
-               fr * r1 * rcap(c) + fw * (w0 + w2) * wcap(c) <= 4
-               fr * r1 * rcap(d) + fw * (w1 + w3) * wcap(d) <= 4
+               fr * r0 / rcap(a) + fw * (w0 + w1) / wcap(a) <= 4
+               fr * r0 / rcap(b) + fw * (w2 + w3) / wcap(b) <= 4
+               fr * r1 / rcap(c) + fw * (w0 + w2) / wcap(c) <= 4
+               fr * r1 / rcap(d) + fw * (w1 + w3) / wcap(d) <= 4
                fr * (1*r0 + 1*r1) + fw * (1*w0 + 1*w1 + 1*w2 + 1*w3) <= 5
         """
         # Create a variable for every read quorum and every write quorum. While
@@ -609,9 +614,7 @@ class Strategy(Generic[T]):
             for x in write_quorum:
                 self.x_write_probability[x] += weight
 
-    @no_type_check
     def __str__(self) -> str:
-        # T may not comparable, so mypy complains about this sort.
         reads = {tuple(sorted(rq)): p for (rq, p) in self.sigma_r.items()}
         writes = {tuple(sorted(wq)): p for (wq, p) in self.sigma_w.items()}
         return f'Strategy(reads={reads}, writes={writes})'
@@ -626,12 +629,12 @@ class Strategy(Generic[T]):
         return self.qs.nodes()
 
     def get_read_quorum(self) -> Set[T]:
-        return set(np.random.choice(list(self.sigma_r.keys()),
-                                    p=list(self.sigma_r.values())))
+        return set(random.choices(list(self.sigma_r.keys()),
+                                  weights=list(self.sigma_r.values()))[0])
 
     def get_write_quorum(self) -> Set[T]:
-        return set(np.random.choice(list(self.sigma_w.keys()),
-                                    p=list(self.sigma_w.values())))
+        return set(random.choices(list(self.sigma_w.keys()),
+                                  weights=list(self.sigma_w.values()))[0])
 
     def load(self,
              read_fraction: Optional[Distribution] = None,
@@ -654,8 +657,6 @@ class Strategy(Generic[T]):
         writes = (1 - fr) * sum(p * len(wq) for (wq, p) in self.sigma_w.items())
         return reads + writes
 
-    # mypy doesn't like calling sum with timedeltas.
-    @no_type_check
     def latency(self,
                 read_fraction: Optional[Distribution] = None,
                 write_fraction: Optional[Distribution] = None) \
@@ -664,16 +665,16 @@ class Strategy(Generic[T]):
         fr = sum(p * fr for (fr, p) in d.items())
 
         reads = fr * sum((
-            p * self.qs._read_quorum_latency({self.node(x) for x in rq})
+            p * self.qs._read_quorum_latency({self.node(x) for x in rq})  # type: ignore[misc]
             for (rq, p) in self.sigma_r.items()
-        ), datetime.timedelta(seconds=0))
+        ), datetime.timedelta(seconds=0))  # type: ignore[arg-type]
 
         writes = (1 - fr) * sum((
-            p * self.qs._write_quorum_latency({self.node(x) for x in wq})
+            p * self.qs._write_quorum_latency({self.node(x) for x in wq})  # type: ignore[misc]
             for (wq, p) in self.sigma_w.items()
-        ), datetime.timedelta(seconds=0))
+        ), datetime.timedelta(seconds=0))  # type: ignore[arg-type]
 
-        return reads + writes
+        return reads + writes  # type: ignore[return-value]
 
     def node_load(self,
                   node: Node[T],
